@@ -109,6 +109,7 @@ SEXP H5Aread_helper_INTEGER(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_
   int b = H5Tget_size(dtype_id);
   H5T_sign_t sgn = H5Tget_sign(dtype_id);
   herr_t herr;
+  int protected = 0;
   
   if((b < 4) | ((b == 4) & (sgn == H5T_SGN_2))) {
       mem_type_id = H5T_NATIVE_INT;
@@ -116,6 +117,7 @@ SEXP H5Aread_helper_INTEGER(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_
       void * buf;
       if (length(_buf) == 0) {
         Rval = PROTECT(allocVector(INTSXP, n));
+        protected++;
         buf = INTEGER(Rval);
       } else {
         buf = INTEGER(_buf);
@@ -124,7 +126,6 @@ SEXP H5Aread_helper_INTEGER(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_
       herr = H5Aread(attr_id, mem_type_id, buf );
       if (length(_buf) == 0) {
         setAttrib(Rval, R_DimSymbol, Rdim);
-        UNPROTECT(1);
       }
   } else if ( ((b == 4) & (sgn == H5T_SGN_NONE)) | (b == 8) ) { 
       // unsigned32-bit or 64-bit integer
@@ -155,6 +156,7 @@ SEXP H5Aread_helper_INTEGER(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_
 
           if (length(_buf) == 0) {
               Rval = PROTECT(allocVector(INTSXP, n));
+              protected++;
               buf = (int *) INTEGER(Rval);
           } else {
               buf = INTEGER(_buf);
@@ -169,6 +171,7 @@ SEXP H5Aread_helper_INTEGER(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_
 
           if (length(_buf) == 0) {
               Rval = PROTECT(allocVector(REALSXP, n));
+              protected++;
               buf = (long long *) REAL(Rval);
           } else {
               buf = REAL(_buf);
@@ -194,24 +197,26 @@ SEXP H5Aread_helper_INTEGER(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_
       
       if (length(_buf) == 0) {
           setAttrib(Rval, R_DimSymbol, Rdim);
-          UNPROTECT(1);
       }
   } else {
       error("Unknown integer type\n");
   }
 
+  UNPROTECT(protected);
   return(Rval);
 }
 
 
 SEXP H5Aread_helper_FLOAT(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t dtype_id) {
   hid_t mem_type_id = -1;
+  int protected = 0;
 
   SEXP Rval;
   mem_type_id = H5T_NATIVE_DOUBLE;
   void * buf;
   if (length(_buf) == 0) {
     Rval = PROTECT(allocVector(REALSXP, n));
+    protected++;
     buf = REAL(Rval);
   } else {
     buf = REAL(_buf);
@@ -225,8 +230,9 @@ SEXP H5Aread_helper_FLOAT(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t
   
   if (length(_buf) == 0) {
     setAttrib(Rval, R_DimSymbol, Rdim);
-    UNPROTECT(1);
   }
+
+  UNPROTECT(protected);
   return(Rval);
 }
 
@@ -271,7 +277,7 @@ SEXP H5Aread_helper_STRING(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t
 SEXP H5Aread_helper_REFERENCE(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t dtype_id) {
   
   void *references;
-  SEXP Rrefs, Rtype, Rval; 
+  SEXP Rrefs, Rtype; 
   
   if(H5Tequal(dtype_id, H5T_STD_REF_OBJ)) {
     Rrefs = PROTECT(allocVector(RAWSXP, sizeof(hobj_ref_t) * n ));
@@ -291,10 +297,13 @@ SEXP H5Aread_helper_REFERENCE(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hi
     return R_NilValue;
   }
   
-  Rval = PROTECT(R_do_new_object(R_getClassDef("H5Ref")));
-  R_do_slot_assign(Rval, mkString("val"), Rrefs);
-  R_do_slot_assign(Rval, mkString("type"), Rtype);
-  UNPROTECT(3);
+  SEXP Rclass = PROTECT(R_getClassDef("H5Ref"));
+  SEXP Rval = PROTECT(R_do_new_object(Rclass));
+  SEXP val = PROTECT(mkString("val"));
+  SEXP type = PROTECT(mkString("type"));
+  R_do_slot_assign(Rval, val, Rrefs);
+  R_do_slot_assign(Rval, type, Rtype);
+  UNPROTECT(6);
   return Rval;
 }
 
@@ -302,16 +311,18 @@ SEXP H5Aread_helper_ENUM(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, hid_t d
   
   SEXP Rval = PROTECT(allocVector(STRSXP, (int) n));
   
-  void *buf = R_alloc(H5Tget_size(dtype_id), n);
+  size_t el_size = H5Tget_size(dtype_id);
+  // We need a pointer to a single byte data type for pointer arithmetic.
+  unsigned char *buf = (unsigned char *) R_alloc(el_size, n);
   H5Aread(attr_id, dtype_id, buf);
-  
+
   size_t max_string_length = 1024;
   char *st = H5allocate_memory(max_string_length, FALSE);
   for (hsize_t i=0; i < n; i++) {
     memset(st, 0, max_string_length);
-    H5Tenum_nameof	(	dtype_id, buf, st, max_string_length);
+    H5Tenum_nameof(dtype_id, buf, st, max_string_length);
     SET_STRING_ELT(Rval, i, mkChar(st));
-    buf += H5Tget_size(dtype_id);
+    buf += el_size;
   }
   H5free_memory(st);
   
@@ -356,8 +367,8 @@ SEXP H5Aread_helper(hid_t attr_id, hsize_t n, SEXP Rdim, SEXP _buf, int bit64con
     Rval = PROTECT(allocVector(REALSXP, n));
     for (hsize_t i=0; i<n; i++) { REAL(Rval)[i] = na; }
     setAttrib(Rval, R_DimSymbol, Rdim);
-    UNPROTECT(1);
     warning("Reading attribute data of type '%s' not yet implemented. Values replaced by NA's.", getDatatypeClass(dtype_id));
+    UNPROTECT(1);
   } break;
   }
 
@@ -371,6 +382,7 @@ SEXP _H5Aread( SEXP _attr_id, SEXP _buf, SEXP _bit64conversion ) {
   hsize_t n = 1;
   SEXP Rdim;
   int bit64conversion = INTEGER(_bit64conversion)[0];
+  int protected = 0;
 
   /***********************************************************************/
   /* attr_id                                                          */
@@ -396,6 +408,7 @@ SEXP _H5Aread( SEXP _attr_id, SEXP _buf, SEXP _bit64conversion ) {
     }
     
     Rdim = PROTECT(allocVector(INTSXP, rank));
+    protected++;
     for (int i = 0; i < rank; i++) {
       INTEGER(Rdim)[i] = dims[i];
     }
@@ -409,9 +422,7 @@ SEXP _H5Aread( SEXP _attr_id, SEXP _buf, SEXP _bit64conversion ) {
   /***********************************************************************/
   SEXP Rval = H5Aread_helper(attr_id, n, Rdim, _buf, bit64conversion);
 
-  if (rank > 0) {
-    UNPROTECT(1);
-  }
+  UNPROTECT(protected);
 
   // close file space
   H5Sclose(file_space_id);
@@ -427,8 +438,6 @@ SEXP _H5Awrite( SEXP _attr_id, SEXP _buf) {
     static const char* H5Ref[] = {"H5Ref", ""};
     int values[3] = {1, 0, NA_LOGICAL};
     
-    int n_unprotect = 0;
-    
     switch(TYPEOF(_buf)) {
     case INTSXP :
         mem_type_id = H5T_NATIVE_INT;
@@ -457,17 +466,21 @@ SEXP _H5Awrite( SEXP _attr_id, SEXP _buf) {
         buf = LOGICAL(_buf);
         break;
     case S4SXP : 
-      if(R_check_class_etc(_buf, H5Ref) >= 0) {
-        if(INTEGER(R_do_slot(_buf, mkString("type")))[0] == H5R_OBJECT) {
+        if(R_check_class_etc(_buf, H5Ref) >= 0) {
+          SEXP typeSlot = PROTECT(mkString("type"));
+          if(INTEGER(R_do_slot(_buf, typeSlot))[0] == H5R_OBJECT) {
             mem_type_id = H5T_STD_REF_OBJ;
-        } else if (INTEGER(R_do_slot(_buf, mkString("type")))[0] == H5R_DATASET_REGION) {
+          } else if (INTEGER(R_do_slot(_buf, typeSlot))[0] == H5R_DATASET_REGION) {
             mem_type_id = H5T_STD_REF_DSETREG;
-        } else {
+          } else {
             mem_type_id = -1;
             Rf_error("Error writing references");
+          }
+          UNPROTECT(1);
         }
-        }
-        buf = RAW(R_do_slot(_buf, mkString("val")));
+        SEXP valSlot = PROTECT(mkString("val"));
+        buf = RAW(R_do_slot(_buf, valSlot));
+        UNPROTECT(1);
         break;
     default :
         mem_type_id = -1;
@@ -479,10 +492,9 @@ SEXP _H5Awrite( SEXP _attr_id, SEXP _buf) {
     herr_t herr = H5Awrite(attr_id, mem_type_id, buf );
     if(herr < 0) { error("Error writing attribute"); }
     SEXP Rval;
-    PROTECT(Rval = allocVector(INTSXP, 1));
-    n_unprotect++;
+    Rval = PROTECT(allocVector(INTSXP, 1));
     INTEGER(Rval)[0] = herr;
-    UNPROTECT(n_unprotect);
+    UNPROTECT(1);
     return Rval;
 }
 
